/***********************************/
/* Source of Views for preferences */
/* of NetCountCalendar             */
/***********************************/

#include <ScrollView.h>
#include <Message.h>
#include <stdio.h>
#include "CalendarViewsPref.h"
#include "CColumnHeader.h"

/*** Messages ***/
const	uint32 U_COLOR			= 'Ucol';
const	uint32 U_ADD			= 'Uadd';
const	uint32 U_MODIFY			= 'Umod';
const	uint32 U_REMOVE			= 'Urem';
const	uint32 U_SELECT			= 'Usel';
const	uint32 U_USECOST		= 'Uuco';
const	uint32 U_USEFIXEDPRICE	= 'Uufp';
const	uint32 U_UPDATE			= 'Uuup';
const	uint32 U_CALENDAR		= 'Uuca';
const	uint32 U_CHANGE_DAY		= 'Uchd';
const	uint32 U_CHANGE_MONTH	= 'Uchm';
const	uint32 U_START_TYPE		= 'Ustt';

// days of months
const int32		MONTHDAYS[12]	= {31,29,31,30,31,30,31,31,30,31,30,31};

// external referencie
extern NetCountCalendarDatas *calendar_datas;

/*************************/
/**** FixedPrice view ****/
/*************************/
FixedPriceViewPref::FixedPriceViewPref(BRect frame,const char *name)
: BView(frame,name,B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	char			tempstring[35];
	int32			h,m,s;
	rgb_color		colorview = {219,219,219,255};
	
	_app = (CounterApp *)be_app;

	SetViewColor(colorview);
	SetFontSize(10.0);

	_useFixedPrice = new BCheckBox(BRect(10,15,120,25),"use-fixed-price",_app->Langage("cvp-use-fixe"),new BMessage(U_USEFIXEDPRICE));
	AddChild(_useFixedPrice);

	h = ((CounterApp *)be_app)->HoursUsed() / 3600;
	m = (((CounterApp *)be_app)->HoursUsed() % 3600) / 60;
	s = ((CounterApp *)be_app)->HoursUsed() % 60;
	
	_tempString.SetTo(_app->Langage("cvp-time-use"));
	sprintf(tempstring,"%02ld:%02ld:%02ld",h,m,s);
	_tempString << tempstring;
	
	_infohours = new BTextView(BRect(145,18,395,38),"info-used-hours",BRect(0,0,250,20),B_FOLLOW_NONE,B_WILL_DRAW);
	_infohours->SetText(_tempString.String());
	_infohours->SetViewColor(colorview);
	_infohours->MakeSelectable(false);
	_infohours->MakeEditable(false);
	AddChild(_infohours);
	
	_fixedPriceBox = new BBox(BRect(10,50,404,205),"fixed-price");
	_fixedPriceBox->SetLabel(_app->Langage("fixe-price"));
	AddChild(_fixedPriceBox);

	_definitionText = new BTextControl(BRect(10,30,375,50),"UDefLabel",_app->Langage("name"),"",NULL);
	_definitionText->SetDivider(35);
	_priceText = new BTextControl(BRect(10,60,130,80),"UDefPrice",_app->Langage("price"),"",NULL);	
	_priceText->SetDivider(35);
	_timeText = new BTextControl(BRect(140,60,375,80),"UDefTime",_app->Langage("cvp-time"),"",NULL);	
	_timeText->SetDivider(150);
	
	_useCalendar = new BCheckBox(BRect(47,80,210,100),"use-fixed-price",_app->Langage("cvp-calendar"),new BMessage(U_CALENDAR));
	_pricePlusText = new BTextControl(BRect(220,80,375,100),"UDefPricePlus",_app->Langage("cvp-add-price"),"",NULL);	
	_pricePlusText->SetDivider(120);
	_chooseSartType = new BCheckBox(BRect(47,100,65,115),"choose-start-type","",new BMessage(U_START_TYPE));
	_startDayText = new BTextControl(BRect(65,100,175,115),"UDefDay",_app->Langage("cvp-start"),"",NULL);
	_startDayText->SetDivider(80);
	_startDay = new CNumberScroll(BRect(176,100,194,118),"UScrollDay",new BMessage(U_CHANGE_DAY),1,31);
	_countEntireMinute = new BCheckBox(BRect(47,120,250,135),"count-entire-minute",_app->Langage("cvp-entire-minute"),NULL);
	
	_updateButton = new BButton(BRect(307,121,377,146),"UUpdate",_app->Langage("button-update"),new BMessage(U_UPDATE));

	_fixedPriceBox->AddChild(_definitionText);
	_fixedPriceBox->AddChild(_priceText);
	_fixedPriceBox->AddChild(_timeText);
	_fixedPriceBox->AddChild(_pricePlusText);
	_fixedPriceBox->AddChild(_useCalendar);
	_fixedPriceBox->AddChild(_updateButton);
	_fixedPriceBox->AddChild(_chooseSartType);
	_fixedPriceBox->AddChild(_startDayText);
	_fixedPriceBox->AddChild(_startDay);
	_fixedPriceBox->AddChild(_countEntireMinute);

	FillWithDatas();
	SetStateFixedPrice();
}

FixedPriceViewPref::~FixedPriceViewPref(void)
{
}

/**** Message Received ****/
void FixedPriceViewPref::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{
		case U_USEFIXEDPRICE :
			SetStateFixedPrice();
			break;
		case U_UPDATE :
			UpdateValues();
			break;
		case U_CALENDAR:
			_pricePlusText->SetEnabled(!_useCalendar->Value());
			break;
		case U_CHANGE_DAY:
			{
			BString startDay;
			int32	value;
			
			msg->FindInt32("_val",&value);
			startDay.SetTo("");
			startDay << value;
			_startDayText->SetText(startDay.String());
			
			}
			break;
		case U_START_TYPE:
			{
				if(_chooseSartType->Value() == B_CONTROL_ON)
					_startDayText->SetLabel(_app->Langage("cvp-type-start"));
				else
					_startDayText->SetLabel(_app->Langage("cvp-start"));				
			}
			break;
		default:
			BView::MessageReceived(msg);
	}
}

/**** Attached to Windows ****/
void FixedPriceViewPref::AttachedToWindow()
{
	_useFixedPrice->SetTarget(this);
	_updateButton->SetTarget(this);
	_useCalendar->SetTarget(this);
	_chooseSartType->SetTarget(this);
	_startDay->SetTarget(this);
	BView::AttachedToWindow();
}

/**** Modify the state of fixed price ****/
void FixedPriceViewPref::SetStateFixedPrice()
{
	bool		fixedpricevalue;
	
	fixedpricevalue = _useFixedPrice->Value();
	calendar_datas->SetUseFixedPrice(fixedpricevalue);

	BMessenger(_app).SendMessage(U_REFRESH_ALL_COUNT);

	_updateButton->SetEnabled(fixedpricevalue);
	_definitionText->SetEnabled(fixedpricevalue);
	_priceText->SetEnabled(fixedpricevalue);
	_timeText->SetEnabled(fixedpricevalue);
	_useCalendar->SetEnabled(fixedpricevalue);
	_pricePlusText->SetEnabled(fixedpricevalue && !_useCalendar->Value());
	_chooseSartType->SetEnabled(fixedpricevalue);
	_startDayText->SetEnabled(fixedpricevalue);
	_startDay->SetEnabled(fixedpricevalue);
	_countEntireMinute->SetEnabled(fixedpricevalue);
}

/**** fill textcontrol with datas ****/
void FixedPriceViewPref::FillWithDatas()
{
	static	char			tempbuffer[20];
	
	_useFixedPrice->SetValue(calendar_datas->UseFixedPrice());
	_definitionText->SetText((calendar_datas->NameForFixedPrice()).String());
	sprintf(tempbuffer,"%.2f",calendar_datas->PriceFixedPrice());
	_priceText->SetText(tempbuffer);
	sprintf(tempbuffer,"%ld",calendar_datas->TimeForFixedPrice());
	_timeText->SetText(tempbuffer);
	sprintf(tempbuffer,"%.2f",calendar_datas->PricePlusFixedPrice());
	_pricePlusText->SetText(tempbuffer);
	_useCalendar->SetValue(calendar_datas->PlusUseCalendar());
	sprintf(tempbuffer,"%ld",calendar_datas->StartDay());
	_startDay->SetValue(calendar_datas->StartDay());
	_startDayText->SetText(tempbuffer);
	_chooseSartType->SetValue(calendar_datas->StartDayType());
	if(calendar_datas->StartDayType() == B_CONTROL_ON)
		_startDayText->SetLabel(_app->Langage("cvp-type-start"));
	_countEntireMinute->SetValue(calendar_datas->EntireMinute());
}

/**** Update values of fixed price ****/
void FixedPriceViewPref::UpdateValues()
{
	int32	time;
	float	price;
	float	priceplus;
	
	sscanf(_timeText->Text(),"%ld",&time);
	sscanf(_priceText->Text(),"%f",&price);
	sscanf(_pricePlusText->Text(),"%f",&priceplus);
	if(time>0)
	{
		calendar_datas->SetNameForFixedPrice(_definitionText->Text());
		calendar_datas->SetPriceFixedPrice(price);
		calendar_datas->SetTimeForFixedPrice(time);
		calendar_datas->SetPricePlusFixedPrice(priceplus);
		calendar_datas->SetPlusUseCalendar(_useCalendar->Value());
		calendar_datas->SetStartDay(_startDay->Value());
		calendar_datas->SetStartDayType(_chooseSartType->Value());
		calendar_datas->SetEntireMinute(_countEntireMinute->Value());
	}
}

/*******************/	
/**** Cost View ****/
/*******************/
ConnectionCostViewPref::ConnectionCostViewPref(BRect frame,const char *name)
: BView(frame,name,B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{	
	_app = (CounterApp *)be_app;

	SetViewColor(219,219,219,255);
	SetFontSize(10.0);

	_useCostConnection = new BCheckBox(BRect(10,15,250,20),"use-cost",_app->Langage("use-cost"),new BMessage(U_USECOST));
	_useCostConnection->SetValue(calendar_datas->UseCostConnection());
	AddChild(_useCostConnection);
	
	_costBox = new BBox(BRect(10,50,404,205),"connection-cost");
	_costBox->SetLabel(_app->Langage("con-cost"));
	AddChild(_costBox);

	_definitionText = new BTextControl(BRect(10,30,375,50),"UDefLabel",_app->Langage("name"),"",NULL);
	_definitionText->SetDivider(35);
	_priceText = new BTextControl(BRect(10,70,120,90),"UDefPrice",_app->Langage("price"),"",NULL);	
	_priceText->SetDivider(35);
	_timeText = new BTextControl(BRect(185,70,375,90),"UDefTime",_app->Langage("cvp-time-sec"),"",NULL);	
	_timeText->SetDivider(115);
	_updateButton = new BButton(BRect(307,121,377,146),"UUpdate",_app->Langage("button-update"),new BMessage(U_UPDATE));

	_costBox->AddChild(_definitionText);
	_costBox->AddChild(_priceText);
	_costBox->AddChild(_timeText);
	_costBox->AddChild(_updateButton);

	
	FillWithDatas();
	SetStateCostConnection();
}

ConnectionCostViewPref::~ConnectionCostViewPref()
{
}

/**** Message Received ****/
void ConnectionCostViewPref::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{
		case U_USECOST :
			SetStateCostConnection();
			break;
		case U_UPDATE :
			UpdateValues();
			break;
		default:
			BView::MessageReceived(msg);
	}
}

/**** Attached to Windows ****/
void ConnectionCostViewPref::AttachedToWindow()
{
	_useCostConnection->SetTarget(this);
	_updateButton->SetTarget(this);
	BView::AttachedToWindow();
}

/**** Modify the state of connection cost ****/
void ConnectionCostViewPref::SetStateCostConnection()
{
	bool	costvalue;
	
	costvalue = _useCostConnection->Value();
	calendar_datas->SetUseCostConnection(costvalue);

	_updateButton->SetEnabled(costvalue);
	_definitionText->SetEnabled(costvalue);
	_priceText->SetEnabled(costvalue);
	_timeText->SetEnabled(costvalue);
}

/**** fill textcontrol with datas ****/
void ConnectionCostViewPref::FillWithDatas()
{
	static	char			tempbuffer[20];
	
	_definitionText->SetText((calendar_datas->NameForCostConnection()).String());
	sprintf(tempbuffer,"%.2f",calendar_datas->PriceOfConnection());
	_priceText->SetText(tempbuffer);
	sprintf(tempbuffer,"%ld",calendar_datas->TimeForPriceOfConnect());
	_timeText->SetText(tempbuffer);
}

/**** Update values of connection cost ****/
void ConnectionCostViewPref::UpdateValues()
{
	int32					timecost;
	float					pricecost;
	
	sscanf(_timeText->Text(),"%ld",&timecost);
	sscanf(_priceText->Text(),"%f",&pricecost);
	if(pricecost>0 && timecost>0)
	{
		calendar_datas->SetNameforCostConnection(_definitionText->Text());
		calendar_datas->SetPriceOfConnection(pricecost);
		calendar_datas->SetTimeForPriceOfConnect(timecost);
	}
}

/***************************/
/**** Connections Views ****/
/***************************/
ConnectionListViewPref::ConnectionListViewPref(BRect frame,const char *name)
: BView(frame,name,B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	_app = (CounterApp *)be_app;

	SetFontSize(10.0);
	SetViewColor(219,219,219,255);
	
	_prefListView = new BListView(BRect(8,25,393,86),"ConnectionListView",B_SINGLE_SELECTION_LIST,B_FOLLOW_LEFT | B_FOLLOW_TOP,B_WILL_DRAW);
	_prefListView->SetFontSize(11.0);
	_prefListView->SetSelectionMessage(new BMessage(U_SELECT));
	AddChild(new BScrollView("scrollConnectionListView",_prefListView,B_FOLLOW_LEFT | B_FOLLOW_TOP,0,false,true));

	_fixColorItem = new BColorControl(BPoint(50,95),B_CELLS_32x8,4,"U_fixColorItem",new BMessage(U_COLOR));
	AddChild(_fixColorItem);
	
	_addButton = new BButton(BRect(175,185,240,210),"UAdd",_app->Langage("button-add"),new BMessage(U_ADD));
	AddChild(_addButton);
	_modifyButton = new BButton(BRect(250,185,315,210),"UModify",_app->Langage("button-modify"),new BMessage(U_MODIFY));
	_modifyButton->SetEnabled(false);
	AddChild(_modifyButton);
	_removeButton = new BButton(BRect(325,185,398,210),"URemove",_app->Langage("button-remove"),new BMessage(U_REMOVE));
	_removeButton->SetEnabled(false);
	AddChild(_removeButton);

	_definitionText = new BTextControl(BRect(5,154,280,174),"UDefLabel",_app->Langage("name"),"",NULL);
	_definitionText->SetDivider(40);
	AddChild(_definitionText);
	_priceText = new BTextControl(BRect(290,154,395,174),"UDefPrice",_app->Langage("price"),"",NULL);
	_priceText->SetDivider(40);
	AddChild(_priceText);	

	LoadListValues();
}

ConnectionListViewPref::~ConnectionListViewPref()
{
	_prefListView->MakeEmpty();
}

/**** Draw method ****/
void ConnectionListViewPref::Draw(BRect frame)
{
	rgb_color	oldcolor;
	rgb_color	oldlowcolor;
	
	oldcolor = HighColor();
	oldlowcolor = LowColor();

	DrawHeader(this,BRect(6,6,49,22),_app->Langage("color"));
	DrawHeader(this,BRect(50,6,319,22),_app->Langage("name"));
	DrawHeader(this,BRect(320,6,408,22),_app->Langage("price"));
	SetLowColor(ViewColor());
	DrawString(_app->Langage("color"),BPoint(12,110));

	SetHighColor(109,109,109);
	StrokeRect(BRect(333,95,368,147));
	SetHighColor(255,255,255);
	StrokeLine(BPoint(334,96),BPoint(367,96));
	StrokeLine(BPoint(334,96),BPoint(334,146));
	SetHighColor(165,165,165);
	StrokeLine(BPoint(367,96),BPoint(367,146));
	StrokeLine(BPoint(334,146),BPoint(367,146));
	SetHighColor(_fixColorItem->ValueAsColor());
	FillRect(BRect(335,97,366,145));
	
	SetLowColor(oldlowcolor);
	SetHighColor(oldcolor);
}

/**** Message Received ****/
void ConnectionListViewPref::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{
		case U_SELECT :
			FillInfoItems();
			break;
		case U_COLOR :
			Invalidate(BRect(333,95,368,147));
			break;
		case U_ADD :
			AddAnItem();
			break;
		case U_REMOVE :
			RemoveAnItem();
			break;
		case U_MODIFY :
			ModifyAnItem();
			break;
		default:
			BView::MessageReceived(msg);
	}
}

/**** Attached to Windows ****/
void ConnectionListViewPref::AttachedToWindow()
{
	BView::AttachedToWindow();
	_prefListView->SetTarget(this);
	_fixColorItem->SetTarget(this);
	_addButton->SetTarget(this);
	_modifyButton->SetTarget(this);
	_removeButton->SetTarget(this);
}

/**** Fill the list with values ****/
void ConnectionListViewPref::LoadListValues()
{
	_prefListView->AddList(calendar_datas->PricesList());
}

/**** Fill EditControl if an item is selected ****/
void ConnectionListViewPref::FillInfoItems()
{
	static	char			currentprice[20];
	int32					selecteditem;
	NetCountCalendarItem	*currentitem;
	
	selecteditem = _prefListView->CurrentSelection();
	if(selecteditem >= 0)
	{
		currentitem = (NetCountCalendarItem*)_prefListView->ItemAt(selecteditem);	
		sprintf(currentprice,"%.2f",currentitem->ItemPrice());
		
		_definitionText->SetText((currentitem->ItemName()).String());
		_priceText->SetText(currentprice);
		_fixColorItem->SetValue(currentitem->ItemColor());
		_modifyButton->SetEnabled(true);
		_removeButton->SetEnabled(!calendar_datas->IsUsed(selecteditem));
	}
	else
	{
		_definitionText->SetText("");
		_priceText->SetText("0.0");
		_fixColorItem->SetValue(UC_BLACK);
		_modifyButton->SetEnabled(false);
		_removeButton->SetEnabled(false);
	}
	Invalidate();
}

/**** Add, Modify or Remove an item ****/
void ConnectionListViewPref::AddAnItem()
{
	float	price;
	BString	name;
	
	name = _definitionText->Text();
	sscanf(_priceText->Text(),"%f",&price);
	if(price>0 && name.CountChars() > 0)
	{
		calendar_datas->AddNetItem(_fixColorItem->ValueAsColor(),name.String(),price);
		_prefListView->AddItem((NetCountCalendarItem*)(calendar_datas->PricesList()->ItemAt(calendar_datas->PricesList()->CountItems()-1)));
		calendar_datas->ChangedList();
	}
}

void ConnectionListViewPref::ModifyAnItem()
{
	NetCountCalendarItem	*currentitem;
	int32					selecteditem;
	float					price;
	
	sscanf(_priceText->Text(),"%f",&price);
	if(price>0)
	{
		selecteditem = _prefListView->CurrentSelection();
		currentitem = (NetCountCalendarItem*)_prefListView->ItemAt(selecteditem);
		
		// update datas
		currentitem->SetItemColor(_fixColorItem->ValueAsColor());
		currentitem->SetItemName((char *)_definitionText->Text());
		currentitem->SetItemPrice(price);

		calendar_datas->ChangedList();
		_prefListView->Invalidate();
	}
}

void ConnectionListViewPref::RemoveAnItem()
{
	int32					selecteditem;
	
	selecteditem = _prefListView->CurrentSelection();

	_prefListView->RemoveItem(selecteditem);
	delete (NetCountCalendarItem*)(calendar_datas->PricesList()->RemoveItem(selecteditem));

	calendar_datas->ChangedList();
}

/*************************/
/**** Blank Day Views ****/
/*************************/
BlankListViewPref::BlankListViewPref(BRect frame,const char *name)
: BView(frame,name,B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	_app = (CounterApp *)be_app;
	_popupprices = NULL;

	SetFontSize(10.0);
	SetViewColor(219,219,219,255);
	_colorBlank = HighColor();
	_indexBlank = -1;
	
	_colorChoice.Set(388,155,408,173);
	
	_blankListView = new BListView(BRect(8,25,393,146),"BlankListView",B_SINGLE_SELECTION_LIST,B_FOLLOW_LEFT | B_FOLLOW_TOP,B_WILL_DRAW);
	_blankListView->SetFontSize(11.0);
	_blankListView->SetSelectionMessage(new BMessage(U_SELECT));
	AddChild(new BScrollView("scrollBlankListView",_blankListView,B_FOLLOW_LEFT | B_FOLLOW_TOP,0,false,true));

	_dayText = new BTextControl(BRect(8,155,57,173),"UDefDay",_app->Langage("day"),"",NULL);
	_dayText->SetDivider(26);
	_dayScroll = new CNumberScroll(BRect(58,155,76,173),"UScrollDay",new BMessage(U_CHANGE_DAY),1,31);
	AddChild(_dayText);
	AddChild(_dayScroll);

	_monthText = new BTextControl(BRect(80,155,139,173),"UDefMonth",_app->Langage("month"),"",NULL);
	_monthText->SetDivider(36);
	_monthScroll = new CNumberScroll(BRect(140,155,158,173),"UScrollMonth",new BMessage(U_CHANGE_MONTH),1,12);
	AddChild(_monthText);
	AddChild(_monthScroll);

	_nameBlank = new BTextControl(BRect(162,155,340,173),"UDefName",_app->Langage("name"),"",NULL);
	_nameBlank->SetDivider(34);
	AddChild(_nameBlank);

	_addButton = new BButton(BRect(175,185,240,210),"UAdd",_app->Langage("button-add"),new BMessage(U_ADD));
	_modifyButton = new BButton(BRect(250,185,315,210),"UModify",_app->Langage("button-modify"),new BMessage(U_MODIFY));
	_modifyButton->SetEnabled(false);
	_removeButton = new BButton(BRect(325,185,398,210),"URemove",_app->Langage("button-remove"),new BMessage(U_REMOVE));
	_removeButton->SetEnabled(false);

	AddChild(_addButton);
	AddChild(_modifyButton);
	AddChild(_removeButton);

	LoadListValues();
	FillInfoItems();

	BuildMenu();
}

BlankListViewPref::~BlankListViewPref(void)
{
	delete _popupprices;
	_blankListView->MakeEmpty();
}

void BlankListViewPref::Draw(BRect frame)
{
	rgb_color	oldcolor;
	rgb_color	oldlowcolor;
	
	oldcolor = HighColor();
	oldlowcolor = LowColor();

	DrawHeader(this,BRect(6,6,54,22),_app->Langage("day"));
	DrawHeader(this,BRect(55,6,109,22),_app->Langage("month"));
	DrawHeader(this,BRect(110,6,349,22),_app->Langage("name"));
	DrawHeader(this,BRect(350,6,408,22),_app->Langage("price"));
	
	SetLowColor(ViewColor());
	DrawString(_app->Langage("price"),BPoint(350,169));

	SetHighColor(109,109,109);
	StrokeRect(_colorChoice);
	SetHighColor(255,255,255);
	StrokeLine(BPoint(_colorChoice.left+1,_colorChoice.top+1),BPoint(_colorChoice.right-1,_colorChoice.top+1));
	StrokeLine(BPoint(_colorChoice.left+1,_colorChoice.top+1),BPoint(_colorChoice.left+1,_colorChoice.bottom-1));
	SetHighColor(165,165,165);
	StrokeLine(BPoint(_colorChoice.right-1,_colorChoice.top+1),BPoint(_colorChoice.right-1,_colorChoice.bottom-1));
	StrokeLine(BPoint(_colorChoice.left+1,_colorChoice.bottom-1),BPoint(_colorChoice.right-1,_colorChoice.bottom-1));
	SetHighColor(_colorBlank);
	FillRect(BRect(_colorChoice.left+2,_colorChoice.top+2,_colorChoice.right-2,_colorChoice.bottom-2));
	
	SetLowColor(oldlowcolor);
	SetHighColor(oldcolor);
}

void BlankListViewPref::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{
		case U_SELECT :
			FillInfoItems();
			break;
		case U_CHANGE_MONTH :
			{
			BString monthString;
			int32	value;
			
			msg->FindInt32("_val",&value);
			monthString.SetTo("");
			monthString << value;
			_monthText->SetText(monthString.String());

			_dayScroll->SetMaxValue(MONTHDAYS[_monthScroll->Value()-1]);
			}
			break;
		case U_CHANGE_DAY :
			{
			BString dayString;
			int32	value;
			
			msg->FindInt32("_val",&value);
			dayString.SetTo("");
			dayString << value;
			_dayText->SetText(dayString.String());
			}
			break;
		case U_ADD :
			AddAnItem();
			break;
		case U_REMOVE :
			RemoveAnItem();
			break;
		case U_MODIFY :
			ModifyAnItem();
			break;
		default:
			BView::MessageReceived(msg);
	}
}

void BlankListViewPref::AttachedToWindow()
{
	BView::AttachedToWindow();
	_blankListView->SetTarget(this);
	_monthScroll->SetTarget(this);
	_dayScroll->SetTarget(this);
	_addButton->SetTarget(this);
	_modifyButton->SetTarget(this);
	_removeButton->SetTarget(this);
}

/**** Fill the list with values ****/
void BlankListViewPref::LoadListValues()
{
	_blankListView->AddList(calendar_datas->BlankDay());
}

/**** Fill EditControl if an item is selected ****/
void BlankListViewPref::FillInfoItems()
{
	int32				selecteditem;
	BString				buffer;
	BlankDayItem		*currentitem;
	
	selecteditem = _blankListView->CurrentSelection();
	if(selecteditem >= 0)
	{
		currentitem = (BlankDayItem*)_blankListView->ItemAt(selecteditem);	
		
		// day update
		_dayScroll->SetValue(currentitem->ItemDay());
		buffer.SetTo("");
		buffer << _dayScroll->Value();
		_dayText->SetText(buffer.String());
		// month update
		_monthScroll->SetValue(currentitem->ItemMonth());
		buffer.SetTo("");
		buffer << _monthScroll->Value();
		_monthText->SetText(buffer.String());
		
		_nameBlank->SetText((currentitem->ItemName()).String());
		_modifyButton->SetEnabled(true);
		_removeButton->SetEnabled(true);
		
		_colorBlank = currentitem->ItemColor();
		_indexBlank = currentitem->ItemIndex();
	}
	else
	{
		_nameBlank->SetText("");
		_dayScroll->SetValue(1);
		_monthScroll->SetValue(1);
		buffer.SetTo("1");
		_dayText->SetText(buffer.String());
		_monthText->SetText(buffer.String());
		_modifyButton->SetEnabled(false);
		_removeButton->SetEnabled(false);
		_colorBlank = UC_BLACK;
		_indexBlank = -1;
	}
	Invalidate();
}

/**** Add, Modify or Remove an item ****/
void BlankListViewPref::AddAnItem()
{
	BString	name;
	
	name = _nameBlank->Text();
	if(_indexBlank >= 0 && name.CountChars() > 0)
	{
		rgb_color	color;
		
		color = ((NetCountCalendarItem*)(calendar_datas->PricesList()->ItemAt(_indexBlank)))->ItemColor();
		calendar_datas->AddBlankItem(_dayScroll->Value(),_monthScroll->Value(),name.String(),_indexBlank,color);
		_blankListView->AddItem((BlankDayItem*)(calendar_datas->BlankDay()->ItemAt(calendar_datas->BlankDay()->CountItems()-1)));
//calendar_datas->ChangedList();
	}
}

void BlankListViewPref::ModifyAnItem()
{
	BString			name;
	
	name = _nameBlank->Text();
	if(_indexBlank >= 0 && name.CountChars() > 0)
	{
		BlankDayItem	*currentitem;
		int32			selecteditem;
		
		selecteditem = _blankListView->CurrentSelection();
		currentitem = (BlankDayItem*)_blankListView->ItemAt(selecteditem);
		
		// update datas
		currentitem->SetItemDay(_dayScroll->Value());
		currentitem->SetItemMonth(_monthScroll->Value());
		currentitem->SetItemName((char *)_nameBlank->Text());
		currentitem->SetItemIndex(_indexBlank);
		currentitem->SetItemColor(_colorBlank);

//calendar_datas->ChangedList();
		_blankListView->Invalidate();
	}
}

void BlankListViewPref::RemoveAnItem()
{
	int32					selecteditem;
	
	selecteditem = _blankListView->CurrentSelection();

	_blankListView->RemoveItem(selecteditem);
	delete (BlankDayItem*)(calendar_datas->BlankDay()->RemoveItem(selecteditem));

//calendar_datas->ChangedList();
}

/**** Set the color and price of ****/
void BlankListViewPref::MouseDown(BPoint Point)
{
	uint32	buttons = 0;

	GetMouse(&Point,&buttons);
	if ((buttons & B_SECONDARY_MOUSE_BUTTON) && _colorChoice.Contains(Point))
	{	
		BuildMenu();
		ConvertToScreen(&Point);
		BMenuItem *selected = _popupprices->Go(Point);
		if (selected)
		{
			_indexBlank = _popupprices->IndexOf(selected);
			_colorBlank = ((NetCountCalendarItem*)(calendar_datas->PricesList()->ItemAt(_indexBlank)))->ItemColor();
			Invalidate(_colorChoice);
		}
	}
	BView::MouseDown(Point);
}

/**** Build the popup menu for prices ****/
void BlankListViewPref::BuildMenu()
{
	NetCountCalendarItem	*priceitem;
	BList					*listofprices;	
	uint32					nbitems;	
	
	if(calendar_datas->ListIsChanged())
	{
		if(_popupprices != NULL)
			delete _popupprices;
	
		_popupprices = new BPopUpMenu("List-Prices");
		listofprices = calendar_datas->PricesList();
		nbitems = listofprices->CountItems();
		for(uint32 i=0;i<nbitems;i++)
		{
			priceitem = (NetCountCalendarItem*)listofprices->ItemAt(i);
			_popupprices->AddItem(new CColorMenuItem(priceitem->ItemColor(), priceitem->ItemName(), NULL));
		}
	}
}
