#ifndef NETCOUNTCALENDARITEM_H
#define NETCOUNTCALENDARITEM_H

#include <List.h>
#include <ListItem.h>
#include <String.h>
#include <Message.h>
#include <interface/GraphicsDefs.h>


// names for messages for NetCountCalendar settings
#define C_NAME_CALENDAR_PREF		"Preferences-NetCountCalendar"
#define	DATA_LIST					"datalist"
#define ITEM_NUMBER					"itemnumber"
#define ITEM_COLOR					"itemcolor"
#define ITEM_PRICE_NAME				"pricename"
#define ITEM_PRICE					"price"
#define WIN_PREF_CALENDAR			"fenetre-pref-netcalendar"
// connection fixed price
#define USE_FIXED_PRICE				"use-fixed-price"
#define NAME_OF_FIXED_PRICE			"name-of-fixed"
#define PRICE_OF_FIXED_PRICE		"price-of-fixed"
#define TIME_OF_FIXED_PRICE			"time-of-fixed"
#define PLUS_PRICE_OF_FIXED_PRICE	"plus-fixed"
#define USES_MONTH_FIXED_PRICE		"uses-moth"
#define USE_CALENDAR_PLUS			"use-calendar-plus"
#define START_DAY_FIXED				"fixed-strat-day"
#define START_DAY_TYPE				"type-strat-day"
#define ENTIRE_MINUTE				"entire-minute"
// connection cost
#define USE_PRICE_OF_CONNECT		"use-price-of-connect"
#define NAME_OF_CONNECT				"name-of-connect"
#define PRICE_OF_CONNECT			"price-of-connect"
#define TIME_OF_CONNECT				"time-of-connect"
// blank day
#define BLANK_ITEM_NUMBER			"blank-item-number"
#define BLANK_ITEM_NAME				"blank-item-name"
#define BLANK_ITEM_DAY				"blank-item-day"
#define BLANK_ITEM_MONTH			"blank-item-month"
#define BLANK_ITEM_INDEX			"blank-item-index"

#define	LENGTH_DATA_LIST	168


/**** classe des items qui composent les connections ****/
class NetCountCalendarItem : public BListItem
{
protected:	
	rgb_color	_colorcase;
	BString		*_nameprice;
	float		_valueprice;

public:
	NetCountCalendarItem(void);
	NetCountCalendarItem(rgb_color,const char *, float);
	virtual ~NetCountCalendarItem(void);
	virtual void DrawItem(BView *owner,BRect frame,bool complete = false);
	
	inline void SetItemColor(rgb_color value)		{ _colorcase = value; }
	inline void SetItemName(const char *value)		{ _nameprice->SetTo(value); }
	inline void SetItemPrice(float value)			{ _valueprice = value; }

	inline rgb_color	ItemColor()		{ return _colorcase; }
	inline BString		ItemName()		{ return *_nameprice; }
	inline float		ItemPrice()		{ return _valueprice; }

};

/**** classe des items pour jour ferie ****/
class BlankDayItem : public BListItem
{
protected:
	int32		_day;
	int32		_month;
	BString		_name;
	int32		_indexPrice;
	rgb_color	_colorCase;
	
public:
	BlankDayItem(void);
	BlankDayItem(int32,int32,const char*,int32,rgb_color);
	virtual ~BlankDayItem(void);

	virtual void DrawItem(BView *owner,BRect frame,bool complete = false);

	inline void SetItemDay(int32 value)			{ _day = value; }
	inline void SetItemMonth(int32 value)		{ _month = value; }
	inline void SetItemName(const char *value)	{ _name.SetTo(value); }
	inline void SetItemIndex(int32 value)		{ _indexPrice = value; }
	inline void SetItemColor(rgb_color value)	{ _colorCase = value; }

	inline int32		ItemDay()		{ return _day; }
	inline int32		ItemMonth()		{ return _month; }
	inline BString		ItemName()		{ return _name; }
	inline int32		ItemIndex()		{ return _indexPrice; }
	inline rgb_color	ItemColor()		{ return _colorCase; }
};

/**** classe des données ****/
class NetCountCalendarDatas
{
protected:
	int32		*_dataslist;
	BList		*_priceslist;	
	BString		*_dayslist;
	BRect		_winframe;
	bool		_itemchanged;
// connection cost
	bool		_useconnectioncost;
	float		_priceofconnect;
	int32		_timeforpriceofconnect;
	BString		_nameconnectioncost;
// fixed price
	bool		_usefixedprice;
	float		_fixedprice;
	int32		_timeforfixedprice;
	BString		_namefixedprice;
	float		_fixedplusprice;
	bool		_plusUseCalendar;
	int32		_startDayType;
	int32		_startDay;
	int32		_entireMinute;
// blank day
	BList		*_blankDay;
	int8		_lastDayChange;
	bool		_isBlankDay;
	int32		_currentIndex;
	
public:
	NetCountCalendarDatas();
	~NetCountCalendarDatas();
	
	void 		SetRowTo(int32,int32);
	void 		SetColTo(int32,int32);
	void		AddNetItem(rgb_color,const char *, float);
	void		AddBlankItem(int32,int32,const char *,int32,rgb_color);
	BMessage	SaveDatas();
	void		LoadDatas(BMessage *);
	void		EraseItem();
	bool		ListIsChanged();
	bool		LoadPricesDatas(char*);
	float		NetItemPrice(tm &);
	rgb_color	NetItemColor(tm &);
	int32		IsBlankDay(tm &);
	bool		IsUsed(int32);

	inline void			ChangedList()					{ _itemchanged = true; }
	inline int32		*DatasList()					{ return _dataslist; }
	inline BList		*PricesList()					{ return _priceslist; }
	inline BList		*BlankDay()						{ return _blankDay; }
	inline BString		DaysList()						{ return *_dayslist; }
	inline BRect		WinFrame()						{ return _winframe; }
	// connection cost
	inline bool			UseCostConnection()				{ return _useconnectioncost; }
	inline float		PriceOfConnection()				{ return _priceofconnect; }
	inline int32		TimeForPriceOfConnect()			{ return _timeforpriceofconnect; }
	inline BString		NameForCostConnection()			{ return _nameconnectioncost; }
	// fixed price
	inline bool			UseFixedPrice()					{ return _usefixedprice; }
	inline float		PriceFixedPrice()				{ return _fixedprice; }
	inline int32		TimeForFixedPrice()				{ return _timeforfixedprice; }
	inline BString		NameForFixedPrice()				{ return _namefixedprice; }
	inline float		PricePlusFixedPrice()			{ return _fixedplusprice; }
	inline bool			PlusUseCalendar()				{ return _plusUseCalendar; }
	inline int32		StartDay()						{ return _startDay; }
	inline int32		StartDayType()					{ return _startDayType; }
	inline int32		EntireMinute()					{ return _entireMinute; }
	
	
	inline void		SetWinFrame(BRect value)						{ _winframe = value; }
	inline void		SetDaysList(const char*value)					{ _dayslist->SetTo(value); }
	// connection cost
	inline void		SetNameforCostConnection(const char *value)		{ _nameconnectioncost.SetTo(value); }	
	inline void		SetPriceOfConnection(float value)				{ _priceofconnect = value; }
	inline void		SetTimeForPriceOfConnect(int32 value)			{ _timeforpriceofconnect = value; }
	inline void		SetUseCostConnection(bool value)				{ _useconnectioncost = value; }
	// fixed price
	inline void		SetNameForFixedPrice(const char *value)			{ _namefixedprice.SetTo(value); }	
	inline void		SetPriceFixedPrice(float value)					{ _fixedprice = value; }
	inline void		SetTimeForFixedPrice(int32 value)				{ _timeforfixedprice = value; }
	inline void		SetUseFixedPrice(bool value)					{ _usefixedprice = value; }
	inline void		SetPricePlusFixedPrice(float value)				{ _fixedplusprice = value; }
	inline void		SetPlusUseCalendar(bool value)					{ _plusUseCalendar = value; }
	inline void		SetStartDay(int32 value)						{ _startDay = value; }
	inline void		SetStartDayType(int32 value)					{ _startDayType = value; }
	inline void		SetEntireMinute(int32 value)					{ _entireMinute = value; }
};
#endif