/*
 *
 *	Counter.cpp
 *
 *  NetCount beta 0.3
 *	
 *  Created : 18/09/1998
 * 
 *  Updated :
 *		09/1998 : Ressources added for the counter (icons)
 *		11/1998 : Detect connection/deconnection automaticaly
 *		12/1999 : NetCount 0.6
 *			- GPL License
 *			- History file is now in the /boot/home/config/settings folder
 *			- Signature is now good : application/x-vnd.mougel.netcount
 *
 *		03/2000 : NetCount 0.8
 *			- settings files are in /boot/home/config/settings/NetCountSetting
 *			- Price of connection
 *			- New windows for history
 *			- Tariff's Calendar
 *			- Remove Connections
 *
 *		06/2000 : NetCount 0.8.1
 *			- Add connections (not possible yet)
 *			- Replicant view in deskbar
 *			- setting for applications
 *			- display total and surf price
 *			- import/export connection from v0.6
 *
 *		07/2000 : NetCount 0.8.2
 *			- new settings for applications
 *			- Multi-langage frensh/english 
 *
 *		09/2000 : NetCount 0.8.3
 *			- start day for fixed price
 *			- separator for import/export connection
 *			- Multi-langage frensh/english & german/italian
 *			- Auto-Backup settings
 *			- Blank day could be used for special prices
 *
 *		09/2000 : NetCount 0.8.4
 *			- fixed some bugs
 *
 *		02/2001 : NetCount 0.8.5
 *			- add options for entire minute in fixed price
 *			- fixed some bugs
 */

#include <Alert.h>
#include <Entry.h>
#include <Path.h>
#include <File.h>
#include <Roster.h>
#include <stdio.h>
#include <stdlib.h>
#include <Deskbar.h>
#include <Control.h>
#include <Screen.h>
#include "Counter.h"
#include "HistoryList.h"
#include "CounterWindow.h"
#include "ReplicantCounter.h"
#include "ProgressWindow.h"
#include "Preferences.h"

// constants for application
const char*		STRING_EXPORT_V081_CST		= "***** Export type V0.8.1 *****";
const int32		MONTHINDEX[12]				= {31,59,90,120,151,181,212,243,273,304,334,365};
const int32		MONTHDAYS[14]				= {31,31,28,31,30,31,30,31,31,30,31,30,31,31};
const char		DELIMITER[3]				= {';',',','\t'};

/**** main(): Application starts here ****/
int main(int argc, char **argv)
{	
	CounterApp	myApplication;

	myApplication.Run();
		
	return(0);
}


/**** CounterApp(): Constructor of NetCount ****/
CounterApp::CounterApp()
: BApplication(APP_SIGNATURE)
{
	BFile			file;
	app_info		info;
	BMessage		application;
	BMessage		netcalendar;

	// initialize ressource file
	GetAppInfo(&info);
	file.SetTo(&(info.ref), B_READ_ONLY);
	resApp.SetTo(&file);

	prefapp = B_ERROR;
	connects = B_ERROR;

	// Create the list for connection items
	connectionlist = new BList();

	// Create datas for the NetCountCalendar
	priceslist = new NetCountCalendarDatas();

	// create a preferences object
	counterpref = new SettingsFile(CPREF_FILE,CPREF_PATH);
	counterconnections = new CPreferences(CPREF_CONN_FILE,CPREF_PATH);
	_langageData = new SettingsFile(CPREF_LANGAGE_FILE,CPREF_PATH);
	
	// Load applications settings
	LoadSettings();

	// load langage
	_langageData->Load();
	if(_langageData->IsEmpty())
		RestoreLangageData();


	// Applications settings
	// default
	wcounter.Set(45,100,200,170);
	_viewDeskbar = B_CONTROL_ON;
	_minimizeAtStart = B_CONTROL_OFF;
	_displayType = B_CONTROL_ON;
	_pulseRate = B_CONTROL_ON;
	if((prefapp = counterpref->Load()) == B_OK)
	{
		BPoint		tempPoint;
		int32		tempInt32;
		
		counterpref->FindMessage(C_NAME_APP_PREF,&application);
		
		// Load preferences
		if(application.FindPoint(APP_WIN_COUNTER,&tempPoint) == B_OK)
			wcounter.Set(tempPoint.x,tempPoint.y,tempPoint.x+155,tempPoint.y+70);

		// readed
		if(application.FindInt32(APP_VIEW_DESKBAR,&tempInt32) == B_OK)
			_viewDeskbar = tempInt32;
		if(application.FindInt32(APP_MINIMIZE_WINDOW,&tempInt32) == B_OK)
			_minimizeAtStart = tempInt32;
		if(application.FindInt32(APP_DISPLAY_TYPE,&tempInt32) == B_OK)
			_displayType = tempInt32;
		if(application.FindInt32(APP_FORCE_RATE,&tempInt32) == B_OK)
			_pulseRate = tempInt32;
		
		// set the default langage
		_displayLangage = 0;
		if(application.FindInt32(APP_LANGAGE_TYPE,&tempInt32) == B_OK)
			_displayLangage = tempInt32;
		SetLangage(_displayLangage);

		priceslist->SetDaysList(Langage("day-list"));

		// Load list of NetCountCalendar datas
		counterpref->FindMessage(C_NAME_CALENDAR_PREF,&netcalendar);	
		priceslist->LoadDatas(&netcalendar);
	}
	else
		(new BAlert("Error","Unable to load settings !","Snif!"))->Go();

	// Load the datas about connections
	LoadConnections();	

	//calcul total time and price
	CalculTotalTime();
	CalculTotalPrice();
	
	// verify the position as screen size
	BScreen		screen;
	BRect		screenSize;
	
	screenSize = screen.Frame();
	_moveWin = !screenSize.Contains(wcounter);
	if(_moveWin)
	{
		wcounter.Set(45,100,200,170);
		priceslist->SetWinFrame(BRect(272,160,690,400));
	}
	
	// creat window	and show it
	aWindow = new CounterWindow(wcounter);
	aWindow->Show();

	// define the default pulse rate
	aWindow->SetPulseRate(250000);

	if(_minimizeAtStart == B_CONTROL_ON)
		aWindow->Minimize(true);
	
	// add replicant
	if(_viewDeskbar == B_CONTROL_ON)	
		AddDeskbarView();
}

CounterApp::~CounterApp()
{
	BMessage netcalendar;

	// lock app
	Lock();
	
	counterpref->MakeEmpty();
	// Save list of NetCountCalendar datas
	netcalendar = priceslist->SaveDatas();
	counterpref->AddMessage(C_NAME_CALENDAR_PREF,&netcalendar);	

	// Auto-Backup
	if(_useBackup == B_CONTROL_ON)
		SaveAuto();

	// save all settings of application
	SaveSettings();
	counterpref->Save();

	// Save the datas about connections
	if(_saveAtEnd == B_CONTROL_OFF)
		SaveConnections();

	// delete the datas for the NetCountCalendar
	delete priceslist;

	while(connectionlist->CountItems())
		delete (HistoryList *)connectionlist->RemoveItem((int32)0);
	delete connectionlist;
	
	delete counterconnections;
	delete counterpref;

	RemoveDeskbarView();

	// lock app
	Unlock();
}

void CounterApp::AddDeskbarView()
{
	//add deskbar view
	BRect buttonRect(0, 0, 49, 15);
	BDeskbar desk;

	if (desk.HasItem("ReplicantCounter"))
		desk.RemoveItem("ReplicantCounter");

	Lock();
	BView *replicantView = new ReplicantCounter(buttonRect, "ReplicantCounter");
	desk.AddItem(replicantView);
	delete replicantView;
	Unlock();
}

void CounterApp::RemoveDeskbarView()
{
	// remove replicant
    BDeskbar desk;

	if (desk.HasItem("ReplicantCounter"))
    	desk.RemoveItem("ReplicantCounter");
}

void CounterApp::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
   	/* BMessenger for dialog with replicant */
	case R_CONNECT_TO_APP_MSG:
		if(message->FindMessenger("replicant-messenger",&_replicantMessenger)==B_OK)
			_replicantMessenger.SendMessage(R_CONNECT_TO_APP_MSG);
		break;
	case R_PULSE_DEFINE_MSG:
		{
  		int32		pulseRate;
  		BMessage	buildMenuMsg(R_BUILD_MENU_MSG);

	   	message->FindInt32("replicant-pulse-rate",&pulseRate);
		if( _pulseRate == B_CONTROL_ON && pulseRate > 250000)
			_replicantMessenger.SendMessage(R_PULSE_DEFINE_MSG);

		buildMenuMsg.AddInt32("type-langage",_displayLangage);
		_replicantMessenger.SendMessage(&buildMenuMsg);
		
		UpdateReplicantTotalTime();
   		}
   		break;
	case R_ABOUT_REQUESTED_MSG:
		aWindow->PostMessage(U_ABOUT);
		break;
	case R_WINDOW_REQUESTED_MSG:
		aWindow->Activate();
		break;
	case U_REFRESH_ALL_COUNT:
		CalculTotalPrice();
		CalculTotalTime();
		aWindow->PostMessage(message);
		break;
	default :
		BApplication::MessageReceived(message);
	}
}

void CounterApp::LoadConnections()
{
	BMessage		NetConnections;
	int32			NbConnections;
	time_t			temptime;	
	tm				*start;
	int32			during;
	float			price;
	bool			fixed;
	ssize_t			tailleobjet;

	// load datas
	counterconnections->Get(CONNECTION_LIST,&NetConnections);

	// connection month for fixed price
	temptime = time(0);
	_usesmonth = *localtime(&temptime);
	FixedMonth(_usesmonth,false);
	FixedMonth(_usesmonth,true);
	if(difftime(temptime,mktime(&_usesmonth))<0)
		FixedMonth(_usesmonth,false);

	
	// for fixed price
	_hoursused = 0;
	
	// all of connection
	NetConnections.FindInt32(CONNECTION_NUMBER,&NbConnections);
	for(int32 i=0;i<NbConnections;i++)
	{
		NetConnections.FindData(CONNECTION_START,B_ANY_TYPE,i,(const void **)&start,&tailleobjet);
		NetConnections.FindInt32(CONNECTION_DURING,i,&during);
		NetConnections.FindFloat(CONNECTION_PRICE,i,&price);
		NetConnections.FindBool(CONNECTION_FIXED,i,&fixed);
		connectionlist->AddItem(new HistoryList(i,start,during,price,DETAILS,fixed));
		if(fixed && difftime(mktime(start),mktime(&_usesmonth))>0)
			_hoursused += during;
	}
	// init the next month
	FixedMonth(_usesmonth,true);
}

void CounterApp::SaveConnections()
{
	BMessage		NetConnections;
	int32			NbConnections;
	tm				NetTime;
	HistoryList		*NetItem;

	NbConnections = connectionlist->CountItems();
	
	// all of connection
	NetConnections.AddInt32(CONNECTION_NUMBER,NbConnections);	
	for(int32 i=0;i<NbConnections;i++)
	{
		NetItem = (HistoryList*)connectionlist->ItemAt(i);	
		NetTime = NetItem->Start();
		NetConnections.AddData(CONNECTION_START,B_ANY_TYPE,&NetTime,sizeof(NetTime));
		NetConnections.AddInt32(CONNECTION_DURING,NetItem->During());
		NetConnections.AddFloat(CONNECTION_PRICE,NetItem->Price());
		NetConnections.AddBool(CONNECTION_FIXED,NetItem->IsFixedPrice());
	}

	// save datas
	counterconnections->Add(CONNECTION_LIST,&NetConnections);
}

/***** Calcul total time *****/
void CounterApp::CalculTotalTime()
{
	int32		nbitems;
	time_t		currentTime_t;
	tm			currentTm;
	tm			conStart;
	BList		*connectionlist;
	
	_totaltime = 0;
	currentTime_t = time(0);
	currentTm = *localtime(&currentTime_t);
	connectionlist = ((CounterApp *)be_app)->connectionlist;
	nbitems = connectionlist->CountItems();
	if(!priceslist->UseFixedPrice())
	{
		for(int32 i=0;i<nbitems;i++)
		{
			conStart = ((HistoryList *)connectionlist->ItemAt(i))->Start();
			if(!(_displayType == B_CONTROL_ON) || (conStart.tm_mon == currentTm.tm_mon && conStart.tm_year == currentTm.tm_year))
				_totaltime += ((HistoryList *)connectionlist->ItemAt(i))->During();
		}
	}
	else
	{
		// calcul total price used for fixed price
		FixedMonth(currentTm,false);
		FixedMonth(currentTm,true);
		if(difftime(currentTime_t,mktime(&currentTm))<0)
			FixedMonth(currentTm,false);
		
		_hoursused = 0;
		for(int32 i=0;i<nbitems;i++)
			if(((HistoryList *)connectionlist->ItemAt(i))->IsFixedPrice())
			{
				conStart = ((HistoryList *)connectionlist->ItemAt(i))->Start();
				if(difftime(mktime(&conStart),mktime(&currentTm))>0)
					_hoursused += ((HistoryList *)connectionlist->ItemAt(i))->During();
				
			}
		_totaltime = _hoursused;
	}
	
	UpdateReplicantTotalTime();
}

/***** Calcul total price *****/
void CounterApp::CalculTotalPrice()
{
	int32		nbitems;
	time_t		currentTime_t;
	tm			currentTm;
	tm			conStart;
	BList		*connectionlist;
	
	_totalprice = 0;
	currentTime_t = time(0);
	currentTm = *localtime(&currentTime_t);
	connectionlist = ((CounterApp *)be_app)->connectionlist;
	nbitems = connectionlist->CountItems();
	if(!priceslist->UseFixedPrice())
	{
		// calcul total price for simple connection
		for(int32 i=0;i<nbitems;i++)
		{
			conStart = ((HistoryList *)connectionlist->ItemAt(i))->Start();
			if(!(_displayType == B_CONTROL_ON) || (conStart.tm_mon == currentTm.tm_mon && conStart.tm_year == currentTm.tm_year))
				_totalprice += ((HistoryList *)connectionlist->ItemAt(i))->Price();
		}
	}
	else
	{
		// calcul total price used for fixed price
		FixedMonth(currentTm,false);
		FixedMonth(currentTm,true);
		if(difftime(currentTime_t,mktime(&currentTm))<0)
			FixedMonth(currentTm,false);
		
		for(int32 i=0;i<nbitems;i++)
			if(((HistoryList *)connectionlist->ItemAt(i))->IsFixedPrice())
			{
				conStart = ((HistoryList *)connectionlist->ItemAt(i))->Start();
				if(difftime(mktime(&conStart),mktime(&currentTm))>0)
					_totalprice += ((HistoryList *)connectionlist->ItemAt(i))->Price();
				
			}
	}
}

/***** Update total time for replicant *****/
void CounterApp::UpdateReplicantTotalTime()
{
	BMessage	message(R_UPDATE_TOTAL_TIME_MSG);

   	message.AddInt32("total-time",_totaltime);
	_replicantMessenger.SendMessage(&message);
}

/**** function to import connections from old version (v0.6) ****/
status_t CounterApp::ImportFile(BEntry *file)
{
	BPath	filePath(file);
	
	BFile	fichier(file, B_READ_ONLY);
	if(fichier.IsReadable())
	{
		int32		nChar;
		off_t		cursPos;
		bool		isHighV06;
		char		tmpChar;
		time_t		temptime;
		tm			tmpTime;
		static char	bufferLine[1000];
		ProgressWindow	*progressWindow;
		int32		nbFileConnection;
		int32		delimiter;
		// for sscanf function
		BString		scanChaine("");
		int32		nbConnection;
		char		dayDate[20];
		char		monthDate[5];
		int32		dayNumber;
		int32		yearNumber;
		int32		hs,ms,ss;
		int32		hd,md,sd;
		char		sp[15];
		tm			monthUsed;
		// for new connection
		int32		NumConnection;
		tm			start;
		int32		during;
		float		price;
		bool		fixed;
		
		temptime = time(0);
		tmpTime = *localtime(&temptime);
		start = tmpTime;
		monthUsed = start;
		FixedMonth(monthUsed,false);
		isHighV06 = false;
		price = 0;
		fixed = false;
		
		if(EraseState())
		{
			while(connectionlist->CountItems())
				delete (HistoryList *)connectionlist->RemoveItem((int32)0);
			
			// for fixed price
			_hoursused = 0;
		}	
		NumConnection = connectionlist->CountItems();		
		
		// count the number of connections in file
		// if export is plus of V0.6, the first line is the version
		// and find the delimiter
		nbFileConnection = 0;
		delimiter = 0;
		nChar = -1;
		cursPos = fichier.Seek(0, SEEK_END);
		fichier.Seek(0, SEEK_SET);
		while(fichier.Position() < cursPos)
		{
			nChar++;
			fichier.Read((void *)(&tmpChar),1);
			bufferLine[nChar] = tmpChar;
			if(tmpChar == '\n')
			{
				bufferLine[nChar] = '\0';
				nChar = -1;
				if(bufferLine[0] == '*')
				{
					if(!isHighV06 && strcmp(bufferLine,STRING_EXPORT_V081_CST) == 0)
						isHighV06 = true;
					else
						sscanf(bufferLine,"***** Delimiter : (%ld) *****",&delimiter);
				}
				else
					nbFileConnection++;
			}
		}

		scanChaine << "%ld = %s %s %ld %ld " << DELIMITER[delimiter];
		scanChaine << " %ld:%ld:%ld " << DELIMITER[delimiter];
		scanChaine << " %ld:%ld:%ld " << DELIMITER[delimiter];
		if(isHighV06)
		{
			scanChaine << " %s " << DELIMITER[delimiter];
			scanChaine << " %d " << DELIMITER[delimiter];
		}
		scanChaine << " ";

		// create and show progress bar
		BRect frameProgress = ((CounterWindow *)aWindow)->FrameWinSetting();

		frameProgress.left += (frameProgress.right - frameProgress.left - 250) / 2;
		frameProgress.top +=  (frameProgress.bottom - frameProgress.top - 50) / 2;
		frameProgress.right = frameProgress.left + 250;
		frameProgress.bottom = frameProgress.top + 50;
		
		progressWindow = new ProgressWindow(frameProgress,nbFileConnection,"Import of Connections");
		progressWindow->Show();
		BMessenger	progressMessenger(progressWindow);
		
		nChar = -1;
		cursPos = fichier.Seek(0, SEEK_END);
		fichier.Seek(0, SEEK_SET);
		while(fichier.Position() < cursPos)
		{
			nChar++;
			fichier.Read((void *)(&tmpChar),1);
			bufferLine[nChar] = tmpChar;
			if(tmpChar == '\n')
			{
				bufferLine[nChar] = '\0';
				if(bufferLine[0] != '*')
				{
					price = 0;
					if(isHighV06)
					{
						sscanf(bufferLine,scanChaine.String(),&nbConnection,dayDate,monthDate,&dayNumber,&yearNumber,&hs,&ms,&ss,&hd,&md,&sd,sp,(int *)&fixed);
						price = atof(sp);
					}
					else
					{
						sscanf(bufferLine,scanChaine.String(),&nbConnection,dayDate,monthDate,&dayNumber,&yearNumber,&hs,&ms,&ss,&hd,&md,&sd);
						fixed = false;
					}

					// initialize the start date
					start.tm_wday = 0;
					for(int32 i=0; i<7;i++)
					{
						tmpTime.tm_wday = i;
						strftime(bufferLine,15,"%A",&tmpTime);
						if(strcmp(dayDate,bufferLine) == 0)
							start.tm_wday = i;					
					}

					start.tm_mon = 0;
					for(int32 i=0; i<12;i++)
					{
						tmpTime.tm_mon = i;
						strftime(bufferLine,5,"%b",&tmpTime);
						if(strcmp(monthDate,bufferLine) == 0)
							start.tm_mon = i;					
					}
					
					start.tm_mday = dayNumber;
					start.tm_year = yearNumber - 1900;
					start.tm_yday = MONTHINDEX[start.tm_mon-1] + dayNumber;
					if(start.tm_mon > 1)
						start.tm_yday += ((yearNumber % 400)==0 || ((yearNumber % 4)==0 && (yearNumber % 100)!=0))?1:0;
				
					// initialize the start time
					start.tm_hour = hs;
					start.tm_min = ms;
					start.tm_sec = ss;
					
					// initialise the during time
					during = hd*3600 + md*60 + sd;

					// recalcul privce for v0.6 import or if you want
					if(!fixed && RecalculTime())
						price = ReCalculPrice(start,during);
					
					// for fixed price
					if(fixed && difftime(mktime(&start),mktime(&monthUsed))>0)
						_hoursused += during;

					// add the connections
					NumConnection++;
					connectionlist->AddItem(new HistoryList(NumConnection,&start,during,price,DETAILS,fixed));

					progressMessenger.SendMessage(U_INC_PROGRESS_BAR);
				}
				nChar = -1;
			}
		}
		//refresh all 
		CalculTotalTime();
		CalculTotalPrice();
		UpdateReplicantTotalTime();
		(BMessenger(aWindow)).SendMessage(U_REFRESH_ALL_COUNT);
		
		_pathImport = filePath.Path();
		_pathImport.RemoveLast(filePath.Leaf());

		progressMessenger.SendMessage(B_QUIT_REQUESTED);
		return B_OK;
	}

	return B_FILE_ERROR;
}

/**** function to export connections (use to backup :-) ****/
status_t CounterApp::ExportFile(BEntry *path,BString *name,int32 at,int32 to)
{
	BPath		filePath;
	BString		fileAndPath;
	static char	dateLine[80];
	
	path->GetPath(&filePath);
	fileAndPath << filePath.Path() << "/" << name->String();
	BFile	fichier(fileAndPath.String(), B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE);
	if (fichier.IsReadable() && fichier.IsWritable())
	{
		int32		nbConnections;
		BString		dateFormat("");
		int32		delimiter;
		bool		exportState = ExportState();
		BString		buffer;
		tm			tmDate;
		int32		h,m,s;
		HistoryList	*current;
		ProgressWindow	*progressWindow;
		
		// number of connections
		nbConnections = to - at + 1; //connectionlist->CountItems();
		
		// create and show progress bar
		BRect frameProgress = wsetting;

		frameProgress.left += (frameProgress.right - frameProgress.left - 250) / 2;
		frameProgress.top +=  (frameProgress.bottom - frameProgress.top - 50) / 2;
		frameProgress.right = frameProgress.left + 250;
		frameProgress.bottom = frameProgress.top + 50;
		
		progressWindow = new ProgressWindow(frameProgress,nbConnections,"Export of Connections");
		progressWindow->Show();
		BMessenger	progressMessenger(progressWindow);

		delimiter = _delimiter;
		fichier.Seek(0, SEEK_SET);
		if(!exportState)
		{
			buffer << STRING_EXPORT_V081_CST << "\n";
			buffer << "***** Delimiter : (" << delimiter << ") *****\n"; 
			fichier.Write((void *)buffer.String(),buffer.CountChars());
		}
		
		dateFormat << "%A %b %d %Y " << DELIMITER[delimiter] << " %H:%M:%S " << DELIMITER[delimiter];
		for(int32 i=at;i<to;i++)
		{
			buffer = "";
			current = (HistoryList *)(connectionlist->ItemAt(i));	
			tmDate = current->Start();
			strftime(dateLine,80,dateFormat.String(),&tmDate);
			buffer << i + 1 << " = " << dateLine;
			h = current->During() / 3600;
			m = (current->During() % 3600) / 60;
			s = current->During() % 60;
			sprintf(dateLine,"%02ld:%02ld:%02ld ",h,m,s);
			buffer << dateLine;
			buffer << DELIMITER[delimiter];
			if(!exportState)
			{
				buffer << " " << current->Price() << " " << DELIMITER[delimiter];
				buffer << " " << (int)current->IsFixedPrice() << " " << DELIMITER[delimiter];
			}
			buffer << " \n";	
			
			fichier.Write((void *)buffer.String(),buffer.CountChars());
			progressMessenger.SendMessage(U_INC_PROGRESS_BAR);
		}
		_pathExport = filePath.Path();

		progressMessenger.SendMessage(B_QUIT_REQUESTED);
		return B_OK;
	}
	
	return B_FILE_ERROR;
}

float CounterApp::ReCalculPrice(tm start,int32 during)
{
	float	total = 0;
	int32	timeForPrice;
	time_t	tempstart;
	time_t	tempstop;
	tm		stop;

	tempstart = mktime(&start);
	if(priceslist->UseCostConnection() == true)
	{
		total = priceslist->PriceOfConnection() * 3600;
		timeForPrice = priceslist->TimeForPriceOfConnect();
		tempstart += timeForPrice;
		during -= timeForPrice;
	}
	start = *localtime(&tempstart);

	if(during>1800) // every 30 minutes, it's for correction of flotting calcul
	{
		int32 nb30Min = during / 1800;
		during -= (1800 * nb30Min);
		for(int32 i=0;i<nb30Min;i++)
		{
			tempstop = mktime(&start) + 1800;
			stop = *localtime(&tempstop);
			total += ((CounterView *)((CounterWindow *)aWindow)->counterView())->ComputeTime(start,stop);
			tempstart += 1800;
			start = *localtime(&tempstart);
		}
	}

	if(during>0)
	{
		tempstop = mktime(&start) + during + 1;
		stop = *localtime(&tempstop);
		total += ((CounterView *)((CounterWindow *)aWindow)->counterView())->ComputeTime(start,stop);
	}
	total /= 3600;

	return total;
}

/**** return the text of a message name for multi-langage ****/
const char	*CounterApp::Langage(const char *dataName)
{
	if(_langageData->FindString(dataName,_displayLangage,&_labelLangage) != B_OK)
		_labelLangage.SetTo("Not Found !");

	return _labelLangage.String();
}

/**** return next or before month date ****/
void CounterApp::FixedMonth(tm &date,bool operation)
{
	if(priceslist->StartDayType() == B_CONTROL_OFF)
		AddSubMonth(date,operation);
	else
		AddSubDays(date,operation);
}

void CounterApp::AddSubMonth(tm &date,bool operation)
{
	time_t	dateTime;
	int32	yearNumber;
	int32	dayNumber;
	int8	addSub;
	int32	diffDay;
	bool	bisextile;
	
	dateTime = mktime(&date);

	// update month and year
	yearNumber = date.tm_year + 1900;
	bisextile = (yearNumber % 400)==0 || ((yearNumber % 4)==0 && (yearNumber % 100)!=0);
	addSub = operation?1:-1;
	dayNumber = MONTHDAYS[date.tm_mon + 1];
	if(!operation)
		dayNumber = MONTHDAYS[date.tm_mon];
	
	if(dayNumber == 28 && bisextile)
		dayNumber = 29;
	
	date.tm_mon++;
	diffDay = priceslist->StartDay();
	if(diffDay>MONTHDAYS[date.tm_mon + addSub])
		diffDay = MONTHDAYS[date.tm_mon + addSub];
	if(diffDay == 28 && bisextile)
		diffDay = 29;
		
	dateTime += ((dayNumber*addSub) - date.tm_mday + diffDay) * 86400;
	date = *localtime(&dateTime);
	date.tm_sec = 0;
	date.tm_min = 0;
	date.tm_hour = 0;	
}

void CounterApp::AddSubDays(tm &date,bool operation)
{
	time_t	dateTime;
	int32	diffDay;
	
	// adjust time
	date.tm_sec = 0;
	date.tm_min = 0;
	date.tm_hour = 0;	

	dateTime = mktime(&date);
	diffDay = priceslist->StartDay();
	if(operation)
		dateTime += (diffDay * 86400);
	else
		dateTime -= (diffDay * 86400);

	date = *localtime(&dateTime);
}

/*** backup auto ****/
void CounterApp::SaveAuto()
{
	tm			current_tm;
	time_t		backup_t;
	time_t		current;
	int8		nbDays;
	int32		at,to;
	BString		fileName("");
	app_info 	info; 
    BPath 		path;
    BEntry		backUpPath;
	
	// fixe variable
	at = 0;
	to = 0;
	current = time(0);
	current_tm = *localtime(&current);
	backup_t = mktime(&_lastBackup);

	// create filename
	fileName << current_tm.tm_mday << "_" << (current_tm.tm_mon + 1) << "_" << (current_tm.tm_year+1900) << ".txt";

	// just day by day
	if(_delayBackup == 0 && _lastBackup.tm_mday >= current_tm.tm_mday)
		return;
		
	// other
	nbDays = (current - backup_t) / 86400;
	if(_delayBackup > 0 && nbDays <= (_delayBackup * 7))
		return;
	
	while(to<connectionlist->CountItems() && at == 0)
	{
		current_tm = ((HistoryList *)(connectionlist->ItemAt(to)))->Start();
		current = mktime(&current_tm);
		if((current - backup_t)>0)
			at = to;
		to++;
	}	
	to = connectionlist->CountItems();

	// Set the good end date backup
	current = time(0);
	_lastBackup = *localtime(&current);

   	//look in app directory for backup
    be_app->GetAppInfo(&info); 
    BEntry entry(&info.ref); 
    entry.GetPath(&path); 
    path.GetParent(&path);
	path.Append("BackUps");
	backUpPath.SetTo(path.Path());
		
	if(backUpPath.Exists())
		ExportFile(&backUpPath,&fileName,at,to);
}

/**** Load application settings ****/
void CounterApp::LoadSettings()
{
	ssize_t		taille_obj;
	BMessage	application;
	// tempory variables
	BPoint		tempPoint;
	tm			*temptm;
	rgb_color	*temp;
	time_t		temp_t;
	
	rgb_color	blanc		= {255,255,255,255};
	rgb_color	noir		= {0,0,0,255};
	rgb_color	vert		= {0,160,0,255};
	rgb_color	vertFonce	= {0,100,50,255};
	rgb_color	bleu		= {0,0,255,255};
	temp_t = 0;

	// read preferences
	if(counterpref->Load() == B_OK)
		counterpref->FindMessage(C_NAME_APP_PREF,&application);	

	// load windows placement
	if(application.FindPoint(APP_WIN_ABOUT,&tempPoint) == B_OK)
		wabout.Set(tempPoint.x,tempPoint.y,tempPoint.x+280,tempPoint.y+150);
	else	
		wabout.Set(150,150,430,300);
			
	if(application.FindPoint(APP_WIN_HISTORY,&tempPoint) == B_OK)
		whistory.Set(tempPoint.x,tempPoint.y,tempPoint.x+375,tempPoint.y+232);
	else
		whistory.Set(210,110,585,342);
	
	if(application.FindPoint(APP_WIN_PRICES,&tempPoint) == B_OK)
		wprice.Set(tempPoint.x,tempPoint.y,tempPoint.x+269,tempPoint.y+260);
	else
		wprice.Set(210,110,479,370);
	
	if(application.FindPoint(APP_WIN_PREF,&tempPoint) == B_OK)
		wsetting.Set(tempPoint.x,tempPoint.y,tempPoint.x+400,tempPoint.y+265);
	else
		wsetting.Set(210,110,610,375);
	
	
	// applications settings
	if(application.FindInt32(APP_SAVE_AT_END,&_saveAtEnd) != B_OK)
		_saveAtEnd = B_CONTROL_OFF;

	// save colors counter view
	if(application.FindData(APP_VIEW_COLOR,B_RGB_COLOR_TYPE,(const void **)&temp,&taille_obj) == B_OK)
		_colorView = *temp;
	else
		_colorView = blanc;
	
	if(application.FindData(APP_TEXT_COLOR,B_RGB_COLOR_TYPE,(const void **)&temp,&taille_obj) == B_OK)
		_colorText = *temp;
	else
		_colorText = noir;

	// load state of history buttons and colors
	if(application.FindInt32(APP_ADD_STATE,&_addState) != B_OK)
		_addState = B_CONTROL_OFF;
	if(application.FindInt32(APP_REMOVE_STATE,&_removeState) == B_OK)
		_removeState = B_CONTROL_ON;
		
	if(application.FindData(APP_FIXED_COLOR,B_RGB_COLOR_TYPE,(const void **)&temp,&taille_obj) == B_OK)
		_colorFixed = *temp;
	else
		_colorFixed = vert;
	if(application.FindData(APP_STAND_COLOR,B_RGB_COLOR_TYPE,(const void **)&temp,&taille_obj) == B_OK)
		_colorStandard = *temp;
	else
		_colorStandard = bleu;
	if(application.FindData(APP_DATE_COLOR,B_RGB_COLOR_TYPE,(const void **)&temp,&taille_obj) == B_OK)
		_colorDate = *temp;
	else
		_colorDate = noir;
	if(application.FindData(APP_TIME_COLOR,B_RGB_COLOR_TYPE,(const void **)&temp,&taille_obj) == B_OK)
		_colorTime = *temp;
	else
		_colorTime = vertFonce;
	if(application.FindData(APP_PRICE_COLOR,B_RGB_COLOR_TYPE,(const void **)&temp,&taille_obj) == B_OK)
		_colorPrice = *temp;
	else
		_colorPrice = bleu;
	
	// load state for old version
	if(application.FindInt32(APP_ERASE_STATE,&_eraseState) != B_OK)
		_eraseState = B_CONTROL_ON;
	if(application.FindInt32(APP_VERSION_STATE,&_exportState) != B_OK)
		_exportState = B_CONTROL_OFF;		
	if(application.FindInt32(APP_RECALCUL_STATE,&_recalculTime) != B_OK)
		_recalculTime = B_CONTROL_OFF;
	if(application.FindInt8(APP_DELIMITER_TYPE,&_delimiter) != B_OK)
		_delimiter = 0;

	// path for import/export
	if(application.FindString(APP_PATH_IMPORT,&_pathImport) != B_OK)
		_pathImport.SetTo("/boot/home/config/settings");
	if(application.FindString(APP_PATH_EXPORT,&_pathExport) != B_OK)
		_pathExport.SetTo("/boot/home/config/settings");



	// backup settings
	if(application.FindInt32(APP_USE_BACKUP,&_useBackup) != B_OK)
		_useBackup = B_CONTROL_ON;
	if(application.FindInt8(APP_DELAY_BACKUP,&_delayBackup) != B_OK)
		_delayBackup = 0;
	if(application.FindData(APP_OLDSTART_BACKUP,B_ANY_TYPE,(const void **)&temptm,&taille_obj) == B_OK)
		_lastBackup = *temptm;
	else
		_lastBackup = *localtime(&temp_t);
		
	if(_moveWin)
	{
		wabout.Set(150,150,430,300);
		whistory.Set(210,110,585,342);
		wprice.Set(210,110,479,370);
		wsetting.Set(210,110,610,375);
	}
}

/**** Save application settings ****/
void CounterApp::SaveSettings()
{
	BMessage	application;
	
	// save data windows
	application.AddPoint(APP_WIN_COUNTER,wcounter.LeftTop());
	application.AddPoint(APP_WIN_ABOUT,wabout.LeftTop());
	application.AddPoint(APP_WIN_HISTORY,whistory.LeftTop());
	application.AddPoint(APP_WIN_PRICES,wprice.LeftTop());
	application.AddPoint(APP_WIN_PREF,wsetting.LeftTop());
	// Applications settings
	application.AddInt32(APP_VIEW_DESKBAR,_viewDeskbar);
	application.AddInt32(APP_MINIMIZE_WINDOW,_minimizeAtStart);
	application.AddInt32(APP_DISPLAY_TYPE,_displayType);
	application.AddInt32(APP_FORCE_RATE,_pulseRate);
	application.AddInt32(APP_SAVE_AT_END,_saveAtEnd);
	// save colors counter view
	application.AddData(APP_VIEW_COLOR,B_RGB_COLOR_TYPE,&_colorView,sizeof(_colorView));
	application.AddData(APP_TEXT_COLOR,B_RGB_COLOR_TYPE,&_colorText,sizeof(_colorText));
	application.AddInt32(APP_LANGAGE_TYPE,_langage);
	// save state of history buttons and colors
	application.AddInt32(APP_ADD_STATE,_addState);
	application.AddInt32(APP_REMOVE_STATE,_removeState);
	application.AddData(APP_FIXED_COLOR,B_RGB_COLOR_TYPE,&_colorFixed,sizeof(_colorFixed));
	application.AddData(APP_STAND_COLOR,B_RGB_COLOR_TYPE,&_colorStandard,sizeof(_colorStandard));
	application.AddData(APP_DATE_COLOR,B_RGB_COLOR_TYPE,&_colorDate,sizeof(_colorDate));
	application.AddData(APP_TIME_COLOR,B_RGB_COLOR_TYPE,&_colorTime,sizeof(_colorTime));
	application.AddData(APP_PRICE_COLOR,B_RGB_COLOR_TYPE,&_colorPrice,sizeof(_colorPrice));
	// save state for old version
	application.AddInt32(APP_ERASE_STATE,_eraseState);
	application.AddInt32(APP_VERSION_STATE,_exportState);
	application.AddInt32(APP_RECALCUL_STATE,_recalculTime);
	application.AddInt8(APP_DELIMITER_TYPE,_delimiter);
	// path for import/export
	application.AddString(APP_PATH_IMPORT,_pathImport);
	application.AddString(APP_PATH_EXPORT,_pathExport);
	// backup settings
	application.AddData(APP_OLDSTART_BACKUP,B_ANY_TYPE,&_lastBackup,sizeof(_lastBackup));
	application.AddInt32(APP_USE_BACKUP,_useBackup);
	application.AddInt8(APP_DELAY_BACKUP,_delayBackup);

	counterpref->AddMessage(C_NAME_APP_PREF,&application);
}

/**** restore datas for langage ****/
status_t CounterApp::RestoreLangageData()
{
	_langageData->MakeEmpty();

	// nothing else in same time
	Lock();
	
	// english
	_langageData->AddString("file","File");
	_langageData->AddString("option","Options");
	_langageData->AddString("about","About...");
	_langageData->AddString("quit","Quit");
	_langageData->AddString("view-connection","History...");
	_langageData->AddString("calendar","Tariff Calendar...");
	_langageData->AddString("setting","Settings...");
	_langageData->AddString("yes","Yes");
	_langageData->AddString("no","No");
	_langageData->AddString("alert-stop","Current connection will not be in the history.\n\nDo you really want to quit ?");
	_langageData->AddString("about-title","About");
	_langageData->AddString("button-great","Great");
	_langageData->AddString("author","Authors :");
	_langageData->AddString("ab-doc","Documentation :");
	_langageData->AddString("thank-bull","Thanks to William Bull for the icon.");
	_langageData->AddString("app-setting","Applications Settings");
	_langageData->AddString("button-add","Add");
	_langageData->AddString("button-remove","Remove");
	_langageData->AddString("button-modify","Modify");
	_langageData->AddString("button-update","Update");
	_langageData->AddString("history-title","History Connections");
	_langageData->AddString("detail-view","View details");
	_langageData->AddString("month-view","View per months");
	_langageData->AddString("total-surf","Total time : ");
	_langageData->AddString("total-price","Total price : ");
	_langageData->AddString("start","Start");
	_langageData->AddString("during","During");
	_langageData->AddString("price","Price");
	_langageData->AddString("for"," for ");
	_langageData->AddString("surf-time","Surf time : ");
	_langageData->AddString("day-list","SunMonTueWedThuFriSat");
	_langageData->AddString("calendar-title","Tariff Calendar");
	_langageData->AddString("tariff-title","Tariff Settings");
	_langageData->AddString("fixe-price","Fixed Prices");
	_langageData->AddString("con-cost","Connection Cost");
	_langageData->AddString("price-list","Prices List");
	_langageData->AddString("cvp-time-use","Time used for this month : ");
	_langageData->AddString("name","Name");
	_langageData->AddString("color","Color");
	_langageData->AddString("cvp-time","Time (in hours)");
	_langageData->AddString("cvp-add-price","Additionnal hour price");
	_langageData->AddString("cvp-use-fixe","Use a Fixed price");
	_langageData->AddString("cvp-time-sec","Time (in second)");
	_langageData->AddString("cvp-calendar","Use tariff Calendar");
	_langageData->AddString("cvp-type-start","Every (day)");
	_langageData->AddString("cvp-start","Start day");
	_langageData->AddString("cvp-entire-minute","Entire Last Minute");
	_langageData->AddString("use-cost","Take connection's cost into account");
	_langageData->AddString("app-pref","Application Settings...");
	_langageData->AddString("app-deskbar","View Deskbar Replicant");
	_langageData->AddString("app-minimize","Minimize window at start");
	_langageData->AddString("app-display","Diplay time and price per month");
	_langageData->AddString("app-rate","Force Pulse Rate to 0.250 sec");
	_langageData->AddString("app-save","Save connection when finish");
	_langageData->AddString("display-pref","Display Settings...");
	_langageData->AddString("display-text","Text color       : ");
	_langageData->AddString("display-back","Background color : ");
	_langageData->AddString("display-exem"," Connections : 12\n Surf time : 00:14:25 for 5.20\n Total : 01:24:10  Price : 37.92\n");
	_langageData->AddString("display-eng","English");
	_langageData->AddString("display-fre","French");
	_langageData->AddString("display-ger","German");
	_langageData->AddString("display-ita","Italian");	
	_langageData->AddString("hist-pref","History Settings...");
	_langageData->AddString("hist-but","Button");
	_langageData->AddString("hist-add","Enable Add Connections");
	_langageData->AddString("hist-rem","Enable Remove Connections");
	_langageData->AddString("hist-std","Standard color :");
	_langageData->AddString("hist-fpr","Fixed Price color :");
	_langageData->AddString("hist-dat","Date color :");
	_langageData->AddString("hist-tim","Time color :");
	_langageData->AddString("hist-pri","Price color :");
	_langageData->AddString("old-pref","Old Version Settings...");
	_langageData->AddString("old-imp","Select the file for import");
	_langageData->AddString("old-exp","Select the file for export");
	_langageData->AddString("old-erase","Erase connections for import");
	_langageData->AddString("old-calcul","Calcul Price in import (v0.6)");
	_langageData->AddString("old-format","Use v0.6 format for export");
	_langageData->AddString("replicant-total","Total time");
	_langageData->AddString("replicant-during","Surf time");
	_langageData->AddString("replicant-show","Show window");
	_langageData->AddString("old-dti","Delimiter :");
	_langageData->AddString("old-def","Default");
	_langageData->AddString("old-com","Comma");
	_langageData->AddString("old-tab","Tabulation");
	_langageData->AddString("backup-pref","Backup Settings...");
	_langageData->AddString("bcp-backup","Backup");
	_langageData->AddString("bcp-28day","28 Days");
	_langageData->AddString("bcp-14day","14 Days");
	_langageData->AddString("bcp-7day","7 Days");
	_langageData->AddString("bcp-day","Days");
	_langageData->AddString("bcp-del","Every : ");
	_langageData->AddString("bcp-use","Use Auto-Backup");
	_langageData->AddString("blank-list","Blank Days");
	_langageData->AddString("total","Total");
	_langageData->AddString("used","Used");
	_langageData->AddString("day","Day");
	_langageData->AddString("month","Month");
	
	//frensh
	_langageData->AddString("file","Fichier");
	_langageData->AddString("option","Options");
	_langageData->AddString("about","A propos...");
	_langageData->AddString("quit","Quitter");
	_langageData->AddString("view-connection","Historique...");
	_langageData->AddString("calendar","Calendrier tarifaire..."	);
	_langageData->AddString("setting","Paramètres...");
	_langageData->AddString("yes","Oui");
	_langageData->AddString("no","Non");
	_langageData->AddString("alert-stop","La connection courante ne sera pas sauvée.\n\nVoulez-vous vraiment quitter NetCount ?");
	_langageData->AddString("about-title","A propos");
	_langageData->AddString("button-great","Bien");
	_langageData->AddString("author","Auteurs :");
	_langageData->AddString("ab-doc","Documentation :");
	_langageData->AddString("thank-bull","Merci a William Bull pour l'icon.");
	_langageData->AddString("app-setting","Paramètres de l'application");
	_langageData->AddString("button-add","Ajouter");
	_langageData->AddString("button-remove","Effacer");
	_langageData->AddString("button-modify","Modifier");
	_langageData->AddString("button-update","Actualiser");
	_langageData->AddString("history-title","Historque des Connections");
	_langageData->AddString("detail-view","Vue en détail");
	_langageData->AddString("month-view","Vue par mois");
	_langageData->AddString("total-surf","Temps total : ");
	_langageData->AddString("total-price","Prix total : ");
	_langageData->AddString("start","Debut");
	_langageData->AddString("during","Durée");
	_langageData->AddString("price","Prix");
	_langageData->AddString("for"," pour ");
	_langageData->AddString("surf-time","Durée : ");
	_langageData->AddString("day-list","DimLunMarMerJeuVenSam");
	_langageData->AddString("calendar-title","Calendrier tarifaire");
	_langageData->AddString("tariff-title","Paramètres tarifaire");
	_langageData->AddString("fixe-price","Forfait");
	_langageData->AddString("con-cost","Connection");
	_langageData->AddString("price-list","Liste des prix");
	_langageData->AddString("cvp-time-use","Temps utilisé ce mois-ci : ");
	_langageData->AddString("name","Nom");
	_langageData->AddString("color","Couleur");
	_langageData->AddString("cvp-time","Temps (en heures)");
	_langageData->AddString("cvp-add-price","Heure supplementaire");
	_langageData->AddString("cvp-use-fixe","Utiliser un forfait");
	_langageData->AddString("cvp-time-sec","Temps (en seconde)");
	_langageData->AddString("cvp-calendar","Utiliser le calendrier des tarifs");
	_langageData->AddString("cvp-type-start","Tous les (jour)");
	_langageData->AddString("cvp-start","Jour de depart");
	_langageData->AddString("cvp-entire-minute","Derniere minute entiere");
	_langageData->AddString("use-cost","Tenir compte du coup de connection");
	_langageData->AddString("app-pref","Application...");
	_langageData->AddString("app-deskbar","Replicant dans la Deskbar");
	_langageData->AddString("app-minimize","Minimiser la fenêtre au demarrage");
	_langageData->AddString("app-display","Affichage temps/prix par mois");
	_langageData->AddString("app-rate","Forcer a rafraichir toutes les 0.25s");
	_langageData->AddString("app-save","Sauver a la déconnection");
	_langageData->AddString("display-pref","Affichage...");
	_langageData->AddString("display-text","Couleur du text  : ");
	_langageData->AddString("display-back","Couleur du fond  : ");
	_langageData->AddString("display-exem"," Connections : 12\n Duree : 00:14:25 pour 5.20\n Total : 01:24:10  Prix : 37.92\n");
	_langageData->AddString("display-eng","Anglais");
	_langageData->AddString("display-fre","Francais");
	_langageData->AddString("display-ger","Allemand");
	_langageData->AddString("display-ita","Italien");
	_langageData->AddString("hist-pref","Historique...");
	_langageData->AddString("hist-but","Bouton");
	_langageData->AddString("hist-add","Autoriser a ajouter une connection");
	_langageData->AddString("hist-rem","Autoriser a effacer des connections");
	_langageData->AddString("hist-std","Couleur standard :");
	_langageData->AddString("hist-fpr","Couleur forfait :");
	_langageData->AddString("hist-dat","Couleur de la date :");
	_langageData->AddString("hist-tim","Couleur du temps :");
	_langageData->AddString("hist-pri","Couleur du prix :");
	_langageData->AddString("old-pref","Anciennes versions...");
	_langageData->AddString("old-imp","Choisir le fichier d'import");
	_langageData->AddString("old-exp","Choisir le fichier d'export");
	_langageData->AddString("old-erase","Effacer les connections");
	_langageData->AddString("old-calcul","Recalcul les prix (import V0.6)");
	_langageData->AddString("old-format","Exporter au format 0.6");
	_langageData->AddString("replicant-total","Temps total");
	_langageData->AddString("replicant-during","Durée");
	_langageData->AddString("replicant-show","Voir fenêtre");
	_langageData->AddString("old-dti","Délimiteur :");
	_langageData->AddString("old-def","Défaut");
	_langageData->AddString("old-com","Virgule");
	_langageData->AddString("old-tab","Tabulation");
	_langageData->AddString("backup-pref","Sauvegarde...");
	_langageData->AddString("bcp-backup","Sauvegarder");
	_langageData->AddString("bcp-28day","28 Jours");
	_langageData->AddString("bcp-14day","14 Jours");
	_langageData->AddString("bcp-7day","7 Jours");
	_langageData->AddString("bcp-day","Jours");
	_langageData->AddString("bcp-del","Tous les : ");
	_langageData->AddString("bcp-use","Utiliser la sauvegarde automatique");
	_langageData->AddString("blank-list","Jours fériées");	
	_langageData->AddString("total","Total");
	_langageData->AddString("used","Utilisé");
	_langageData->AddString("day","Jour");
	_langageData->AddString("month","Mois");

	// German
	_langageData->AddString("file","Datei");
	_langageData->AddString("option","Optionen");
	_langageData->AddString("about","Über...");
	_langageData->AddString("quit","Beenden");
	_langageData->AddString("view-connection","Verlauf...");
	_langageData->AddString("calendar","Tarif-Kalender...");
	_langageData->AddString("setting","Einstellungen...");
	_langageData->AddString("yes","Ja");
	_langageData->AddString("no","Nein");
	_langageData->AddString("alert-stop","Aktuelle Verbindung noch nicht im Verlauf gesichert.\n\nWirklich beenden ?");
	_langageData->AddString("about-title","Über");
	_langageData->AddString("button-great","Großartig");
	_langageData->AddString("author","Autoren :");
	_langageData->AddString("ab-doc","Dokumentation :");
	_langageData->AddString("thank-bull","Dank an William Bull für das Icon.");
	_langageData->AddString("app-setting","Programm-Einstellungen");
	_langageData->AddString("button-add","Hinzufügen");
	_langageData->AddString("button-remove","Entfernen");
	_langageData->AddString("button-modify","Ändern");
	_langageData->AddString("button-update","Aktualisieren");
	_langageData->AddString("history-title","Gemachte Verbindungen");
	_langageData->AddString("detail-view","Ansicht im Detail");
	_langageData->AddString("month-view","Ansicht pro Monat");
	_langageData->AddString("total-surf","Gesamtzeit : ");
	_langageData->AddString("total-price","Gesamtkosten : ");
	_langageData->AddString("start","Start");
	_langageData->AddString("during","Dauer");
	_langageData->AddString("price","Kosten");
	_langageData->AddString("for"," für ");
	_langageData->AddString("surf-time","Online-Zeit : ");
	_langageData->AddString("day-list","SonMonDieMitDonFreSam");
	_langageData->AddString("calendar-title","Tarif-Kalender");
	_langageData->AddString("tariff-title","Tarif Einstellungen");
	_langageData->AddString("fixe-price","Festpreise");
	_langageData->AddString("con-cost","Verbindungskosten");
	_langageData->AddString("price-list","Preis-Liste");
	_langageData->AddString("cvp-time-use","Verbrauchte Zeit für diesen Monat : ");
	_langageData->AddString("name","Name");
	_langageData->AddString("color","Farbe");
	_langageData->AddString("cvp-time","Zeit (in Stunden)");
	_langageData->AddString("cvp-add-price","Preis für zusätzl. Stunden");
	_langageData->AddString("cvp-use-fixe","Festpreis benutzen");
	_langageData->AddString("cvp-time-sec","Zeit (in Sekunden)");
	_langageData->AddString("cvp-calendar","Benutze Tarif-Kalender");
	_langageData->AddString("cvp-type-start","Alle (Tag)");
	_langageData->AddString("cvp-start","Abreise Tag");
	_langageData->AddString("cvp-entire-minute","Minutengenau");
	_langageData->AddString("use-cost","Verbindungskosten mit einbeziehen");
	_langageData->AddString("app-pref","Programm-Einstellungen...");
	_langageData->AddString("app-deskbar","Replicant in der Deskbar");
	_langageData->AddString("app-minimize","Fenster beim Start minimieren");
	_langageData->AddString("app-display","Zeit und Kosten pro Monat anzeigen");
	_langageData->AddString("app-rate","Erzwinge Puls-Rate mit 0.250 sec");
	_langageData->AddString("app-save","Verbindung bei Ende speichern");
	_langageData->AddString("display-pref","Einstellungen der Ansicht...");
	_langageData->AddString("display-text","Textfarbe       : ");
	_langageData->AddString("display-back","Hintergrundfarbe : ");
	_langageData->AddString("display-exem"," Verbindungen : 12\n Surf time : 00:14:25 for 5.20\n Gesamt : 01:24:10  Kosten : 37.92\n");
	_langageData->AddString("display-eng","English");
	_langageData->AddString("display-fre","French");
	_langageData->AddString("display-ger","Deutsch");
	_langageData->AddString("display-ita","Italienisch");
	_langageData->AddString("hist-pref","Einstellungen des Verlaufs...");
	_langageData->AddString("hist-but","Button");
	_langageData->AddString("hist-add","Aktiviere Verbindungen hinzufügen");
	_langageData->AddString("hist-rem","Aktiviere Verbindungen entfernen");
	_langageData->AddString("hist-std","Standardfarbe :");
	_langageData->AddString("hist-fpr","Farbe für Festpreis :");
	_langageData->AddString("hist-dat","Farbe für Datum :");
	_langageData->AddString("hist-tim","Farbe für Zeit :");
	_langageData->AddString("hist-pri","Farbe für Kosten :");
	_langageData->AddString("old-pref","Einstellungen der alten Version...");
	_langageData->AddString("old-imp","Wähle die Datei für den Import");
	_langageData->AddString("old-exp","Wähle die Datei für den Export");
	_langageData->AddString("old-erase","Lösche Verbindungen für den Import");
	_langageData->AddString("old-calcul","Berechne Kosten im Import (v0.6)");
	_langageData->AddString("old-format","Benutze Format v0.6 für Export");
	_langageData->AddString("replicant-total","Gesamtzeit");
	_langageData->AddString("replicant-during","Online-Zeit");
	_langageData->AddString("replicant-show","Zeige Fenster");
	_langageData->AddString("old-dti","Delimiter :");
	_langageData->AddString("old-def","Default");
	_langageData->AddString("old-com","Comma");
	_langageData->AddString("old-tab","Tabulation");
	_langageData->AddString("backup-pref","Backup Einstellungen...");
	_langageData->AddString("bcp-backup","Backup");
	_langageData->AddString("bcp-28day","28 Tage");
	_langageData->AddString("bcp-14day","14 Tage");
	_langageData->AddString("bcp-7day","7 Tage");
	_langageData->AddString("bcp-day","Tage");
	_langageData->AddString("bcp-del","Alle : ");
	_langageData->AddString("bcp-use","Benutze Auto-Backup");
	_langageData->AddString("blank-list","Offene Tage");
	_langageData->AddString("total","Gesamt");
	_langageData->AddString("used","Benutzt");
	_langageData->AddString("day","Tag");
	_langageData->AddString("month","Monat");
	
	// italiano / italien
	_langageData->AddString("file","File");
	_langageData->AddString("option","Opzioni");
	_langageData->AddString("about","About...");
	_langageData->AddString("quit","Uscita");
	_langageData->AddString("view-connection","Storia");
	_langageData->AddString("calendar","Tariffe");
	_langageData->AddString("setting","Impostazioni");
	_langageData->AddString("yes","Si");
	_langageData->AddString("no","No");
	_langageData->AddString("alert-stop","I dati della connessione corrente saranno persi\n\nVuoi uscire ugualmente?");
	_langageData->AddString("about-title","About");
	_langageData->AddString("button-great","Grande");
	_langageData->AddString("author","Autori:");
	_langageData->AddString("ab-doc","Documentazione:");
	_langageData->AddString("thank-bull","Un grazie a William Bull per l'icona.");
	_langageData->AddString("app-setting","Impostazioni dell'applicazione");
	_langageData->AddString("button-add","aggiungi");
	_langageData->AddString("button-remove","rimuovi");
	_langageData->AddString("button-modify","modifica");
	_langageData->AddString("button-update","aggiorna");
	_langageData->AddString("history-title","Storia delle connessioni");
	_langageData->AddString("detail-view","dettagli");
	_langageData->AddString("month-view","per mese");
	_langageData->AddString("total-surf","Tempo totale: ");
	_langageData->AddString("total-price","Prezzo totale: ");
	_langageData->AddString("start","inizio");
	_langageData->AddString("during","durata");
	_langageData->AddString("price","prezzo");
	_langageData->AddString("for"," per ");
	_langageData->AddString("surf-time","tempo : ");
	_langageData->AddString("day-list","DomLunMarMerGioVenSab");
	_langageData->AddString("calendar-title","Calendario delle tariffe");
	_langageData->AddString("tariff-title","Impostazione tariffe");
	_langageData->AddString("fixe-price","Prezzo fisso");
	_langageData->AddString("con-cost","Scatto alla risposta");
	_langageData->AddString("price-list","Lista tariffe");
	_langageData->AddString("cvp-time-use","Tempo utilizzato questo mese: ");
	_langageData->AddString("name","Nome");
	_langageData->AddString("color","Colore");
	_langageData->AddString("cvp-time","Tempo (in ore)");
	_langageData->AddString("cvp-add-price","Prezzo ulteriori ore");
	_langageData->AddString("cvp-use-fixe","Usa prezzo fisso");
	_langageData->AddString("cvp-time-sec","tempo (in secondi)");
	_langageData->AddString("cvp-calendar","Usa calendario delle tariffe");
	_langageData->AddString("cvp-type-start","Ogni (Giorno)");
	_langageData->AddString("cvp-start","Giorno d'inizio");
	_langageData->AddString("cvp-entire-minute","Arrotonda Ultimo Minuto");
	_langageData->AddString("use-cost","Considera costo dello scatto");
	_langageData->AddString("app-pref","Impostazioni dell'applicazione");
	_langageData->AddString("app-deskbar","Vedi Deskbar Replicant");
	_langageData->AddString("app-minimize","Minimizza finestra all'avvio");
	_langageData->AddString("app-display","Visualizza tempo e prezzo per mese");
	_langageData->AddString("app-rate","Forza Pulse Rate a 0.250 sec");
	_langageData->AddString("app-save","Salva la connessione quando onclusa");
	_langageData->AddString("display-pref","Impostazioni di visualizzazione");
	_langageData->AddString("display-text","Colore del testo: ");
	_langageData->AddString("display-back","Colore dello sfondo: ");
	_langageData->AddString("display-exem"," Connessioni : 12\n tempo : 00:06:06 per 5.20\n Totale: 01:24:10  Prezzo: 37.92\n");
	_langageData->AddString("display-eng","Inglese");
	_langageData->AddString("display-fre","Francese");
	_langageData->AddString("display-ger","Tedesco");
	_langageData->AddString("display-ita","Italian");
	_langageData->AddString("hist-pref","Impostazioni della storia");
	_langageData->AddString("hist-but","Bottone");
	_langageData->AddString("hist-add","Abilita aggiunta connessioni");
	_langageData->AddString("hist-rem","Abilita rimozione connessioni");
	_langageData->AddString("hist-std","Colori standard:");
	_langageData->AddString("hist-fpr","Colore del prezzo fisso:");
	_langageData->AddString("hist-dat","Colore data:");
	_langageData->AddString("hist-tim","Colore ora:");
	_langageData->AddString("hist-pri","Colore prezzo:");
	_langageData->AddString("old-pref","Impostazioni vecchie versioni");
	_langageData->AddString("old-imp","Seleziona il file da importare");
	_langageData->AddString("old-exp","Seleziona il file da esportare");
	_langageData->AddString("old-erase","Elimina connessioni per importo");
	_langageData->AddString("old-calcul","Calcola prezzo nell'importazione (v0.6)");
	_langageData->AddString("old-format","Usa formato v0.6  per l'esportazione");
	_langageData->AddString("replicant-total","Tempo totale");
	_langageData->AddString("replicant-during","Tempo connessione");
	_langageData->AddString("replicant-show","Visualizza finestra");
	_langageData->AddString("old-dti","Delimitatore:");
	_langageData->AddString("old-def","Default");
	_langageData->AddString("old-com","Virgola");
	_langageData->AddString("old-tab","Tabulazione");
	_langageData->AddString("backup-pref","Salva impostazioni...");
	_langageData->AddString("bcp-backup","Salvataggio");
	_langageData->AddString("bcp-28day","28 Giorno");
	_langageData->AddString("bcp-14day","14 Giorno");
	_langageData->AddString("bcp-7day","7 Giorno");
	_langageData->AddString("bcp-day","Giorno");
	_langageData->AddString("bcp-del","Ogni : ");
	_langageData->AddString("bcp-use","Utilizza il salvataggio automatico");
	_langageData->AddString("blank-list","Giorni Feriali");
	_langageData->AddString("total","Totale");
	_langageData->AddString("used","Utilizzato");
	_langageData->AddString("day","Giorno");
	_langageData->AddString("month","Mese");


	// save for restore
	_langageData->Save();

	// nothing else in same time
	Unlock();
	
	return B_OK;
}
